clear

% where are my custom functions?
addpath('../routines');


% ---   parameters for system size   ---
% parameter to generate system size n=2^k-1
k=menu('Problemgre, Testbeispiele',...
       '  15',...
       '  31',...
       '  63',...
       ' 127',...
       ' 255',...
       ' 511',...
       '1023',...
       '2047',...
       '4095',...
       '8192')+4;

% system size
n=2^k-1;
% --------------------------------------


% ---   parameters for multigrid cycle   ---
% number of levels, <=k
level=k-(k-5);
% ------------------------------------------




nmb=9;
strg=cell(nmb,1);
fun=cell(nmb,1);
strg{1}='laplace 1-D ';                                fun{1}='fun1';
strg{2}='laplace 1-D, alpha=1, beta=100';              fun{2}='fun2';
strg{3}='laplace 1-D, 1+sin(8pix)^2';                  fun{3}='fun3';
strg{4}='laplace 1-D, 1+sin(16pix)^2';                 fun{4}='fun4';
strg{5}='laplace 1-D, 1+sin(32pix)^2';                 fun{5}='fun5';
strg{6}='laplace 1-D, 1+e^{pix}sin(8pix)^2';           fun{6}='fun6';
strg{7}='laplace 1-D, 1+e^{2pix}sin(8pix)^2';          fun{7}='fun7';
strg{8}='laplace 1-D, jump. coeff [1,100,1e3,1e4]';    fun{8}='fun9';

strg{nmb}='END';



choice=menu('choice of sample problems',strg);...

if choice<nmb

   fprintf('%s\n',strg{choice});
   fprintf('problem size n=%5d\n',n);
   fprintf('number of levels: %d\n',level);

   if choice<=9
      IsSPD=-1;
   else
      IsSPD=0;
   end % if-else
   
   global Alist Plist Dlist GSFlist GSBlist AR Ap;
   % list of matrices for any level
   Alist=cell(1,level);
   % projection (prolongation and restriction^T)
   Plist=cell(1,level-1);

   % depending on the problem generate initial matrix
   Alist{1}=lap1d(n,fun{choice}); 

   % direct solver using symmetric minimum degree ordering
   flpdirect=flops;
   Ap=symmmd(Alist{1});
   AR=chol(Alist{1}(Ap,Ap));
   flpdirect=flops-flpdirect;
   
   % apply multigrid to the diagonal preconditioned system
   % in this case the averageing of the entries has to be adapted
   % D0=spdiags(1./sqrt(diag(Alist{1})),0,n,n);
   % Alist{1}=D0'*Alist{1}*D0;



   % --------------   Generation of the multilevel structure   --------------
   % dampening parameter for diagonal smoothing
   flpmg=flops;
   omegaD=2/3;
   nn=n;
   l=n+1;
   for m=1:level-1   
       % standard diagonal preconditioning
       Dlist{m}=omegaD*spdiags(1./diag(Alist{m}),0,nn,nn);
       GSFlist{m}=tril(Alist{m});
       GSBlist{m}=triu(Alist{m});
    
       % projection operator
       buff=spdiags(ones(nn,1)*[1/2 1 1/2],-1:1,nn,nn); 
       % D=spdiags(1./sqrt(diag(Alist{m})),0,nn,nn);
       % buff=speye(nn)-(1/2)*D'*Alist{m}*D; % with init. prec.
       % for ll=2:2:nn
       %     buff(:,ll)=buff(:,ll)/sum(buff(:,ll));
       % end % for
       Plist{m}=buff(:,2:2:nn);
       
       l=l/2;
       nn=nn-l;
       
       % coarse grid system
       % Galerking-type
       Alist{m+1}=Plist{m}'*Alist{m}*Plist{m};
       % Geometric type
       % Alist{m+1}=lap1d(nn,fun{choice}); 

   end % for m
   Dlist{level}.U=chol(Alist{level});Dlist{level}.L=Dlist{level}.U';
   GSFlist{level}.L=Dlist{level}.L;
   GSFlist{level}.U=Dlist{level}.U;
   flpmg=flops-flpmg;
   % ------------   END Generation of the multilevel structure   ------------

   
   % call script file with iteration process
   iterationirl1;

end % if choice<3
