clear
addpath('../routines');
loadmat;

choice=menu('Symmetrische Umordnung des Systems',...
            'keine',...
	    'Reverse Cuthill-McKee',...
	    'Minimum Degree');
	   
if choice==2
   p=symrcm(A);
   A=A(p,p);
   fprintf('Matrix umgeordnet mittels `symrcm`\n');
elseif choice==3
   p=symmmd(A);
   A=A(p,p);
   fprintf('Matrix umgeordnet mittels `symmmd`\n');
end % if-elseif

n=size(A,1);
DSCALE=speye(n);
for i=1:n
    DSCALE(i,i)=1/norm(A(i,:),1);
end % for i
A=DSCALE*A;
fprintf('diagonal scaling used\n');

droptol=menu('Wahl der Schwelle',...
             '0.1',...
             '0.01',...
             '0.001',...
             '0.0001',...
             '0.00001',...
             '0.000001');

PARAM.droptol=10^(-droptol);
PARAM.tresh  =0.1;
PARAM.udiag  =0;
PARAM.milu   =0;



flp=flops;
[L,U]=luinc(A,PARAM);
fprintf('LUINC, flops %8.1e \n',flops-flp);
fprintf('Fill-in Faktor: %4.1f\n',(nnz(L)+nnz(U)-n)/nnz(A));

fprintf('Fill-in L: %5d(%4.1f av.), U: %5d(%4.1f av.), insgesamt %5d(%4.1f av.)\n\n',...
        nnz(L)-n,nnz(L)/n-1,...
        nnz(U)-n,nnz(U)/n-1,...
        nnz(L)+nnz(U)-n,nnz(L)/n+nnz(U)/n-1);
     

% right hand side
x=ones(n,1);
b=A*x;

% ------------------------   Apply GMRES   -----------------------   
cnd=condest(spdiags(diag(U),0,n,n));
if 1/cnd<.001*eps
   fprintf('\nDiagonale von U ist hchstwahrscheinlich singulr(cond ~%8.1e)\n\n',cnd);
   fprintf('LUINC,   keine Iteration durchgefhrt\n');
else
   strg=cell(6,1);
   strg{1}='BICG';
   strg{2}='CGS';
   strg{3}='BICGSTAB';
   strg{4}='GMRES';
   strg{5}='QMR';
   strg{6}='ENDE';
   tol=sqrt(eps);

   type_iteration=0;
   while type_iteration<6
         type_iteration=menu('Auswahl der Iterationsverfahren',strg);
         
         if type_iteration<6
            fprintf('%s\n',strg{type_iteration});
            fprintf('norm of the initial residual %8.1e\n',norm(b,2));
            flp=flops;
         end
         if type_iteration==1 
            y=bicg(A,b,tol,n,L,U);
         elseif type_iteration==2
            y=cgs(A,b,tol,n,L,U);
         elseif type_iteration==3
            y=bicgstab(A,b,tol,n,L,U);
         elseif type_iteration==4
            RST=10*(menu('Restartlnge',...
                         'kein Restart',...
                         '10',...
                         '20',...
                         '30',...
                         '40')-1);
            if RST==0
	       RST=n;
            end
            y=gmres(A,b,RST,tol,n,L,U);
         elseif type_iteration==5
            y=qmr(A,b,tol,n,L,U);
         end

         if type_iteration<6
            fprintf('norm of the final residual   %8.1e\n',norm(b-A*y,2));
            fprintf('norm of the final error      %8.1e\n',norm(x-y,2));
            fprintf('flops %8.1e\n\n',flops-flp);
         end
   end % while
end
% ----------------------   END Apply GMRES   ---------------------

return   

