function [P,C]=COARSEB(A,coord)
% [P,C]=COARSEB(A,coord)
%
% AMG by Braess
% give a matrix A a rectangular coarse grid projection P matrix is
% constructed

n=size(A,1);

if nargin==2
   M=[];
   clf;
   xmin=min(coord(:,1))-1; xmax=max(coord(:,1))+1;
   ymin=min(coord(:,2))-1; ymax=max(coord(:,2))+1;
   gplot(A,coord,'-r');hold on;
   axis([xmin xmax ymin ymax]);
   pause(.001)
end


% -----------------   first step: building pairs   ------------------------
% set of pairs
P=cell(1,n); 
% how many pairs do exist
p=0;
ndetermined=0;
% list of undetermined nodes
undetermined=1:n;
% list of check marks. Let k=isdetermined(i). If k<0, then i has not been 
% determined yet. In this case it ist number |k| in the list of undetermined
% nodes. If k>0, then i is a member of group k
isdetermined=-undetermined;

% final entry in list undetermined, that is a neighbour of pairs
% that have already been selected
nnbp=0;

% as long as nodes are available
while ndetermined<n
      p=p+1;
      % choose an undetermined node i
      i=undetermined(1);
      % list might be inconsistent
      while isdetermined(i)>0
            undetermined(1)=undetermined(end);
            undetermined(end)=[];
            i=undetermined(1);
      end % while
      % choose the first member
      P{p}=i;
      if nargin==2
         gplot(1,coord(i,:),'ob');hold on
         axis([xmin xmax ymin ymax]);
         pause(.001);
      end
      % remove i from the list of undetermined nodes
      ndetermined=ndetermined+1;
      if nnbp
         % move final neighbouring node to the front
         undetermined(1)=undetermined(nnbp);
         isdetermined(undetermined(1))=-1;
         % move last node to position nnbp
         undetermined(nnbp)=undetermined(end);
         undetermined(end)=[];
         if nnbp<=length(undetermined)
            isdetermined(undetermined(nnbp))=-nnbp;
         end % i
         nnbp=nnbp-1;
      else
         undetermined(1)=undetermined(end);
         undetermined(end)=[];
         if ~isempty(undetermined)
            isdetermined(undetermined(1))=-1;
         end % if
      end % if
      isdetermined(i)=p;

      % set of undetermined neighbours of i
      nb=find(A(i,:));
      % neighbours belonging to previous pairs
      nbi=setdiff(nb,i);
      nbi=nbi(find(isdetermined(nbi)>0));
      nbi=isdetermined(nbi);
      % neighbours NOT belonging to previous pairs
      nb=nb(find(isdetermined(nb)<0));
      % shuffle neighbours of i to the end of list 'undetermined'
      for j=nb
          % swap entries
          posj=-isdetermined(j);
          if posj>nnbp
             nnbp=nnbp+1;
             undetermined(posj)=undetermined(nnbp);
             undetermined(nnbp)=j;
             isdetermined(undetermined(posj))=-posj;
             isdetermined(j)=-nnbp;             
          end
      end
      % i is a singleton
      if isempty(nb)
         % do nothing
      else
         % maximize A(i,j)*A(i,j)/(A(i,i)*A(j,j)) over all neighbours j of i
         rnb=-1;
         isneighbour=0;
         issameneighbour=0;
         for jj=1:length(nb)
             j_new=nb(jj);
             rnb_new=A(i,j_new)*A(i,j_new)/(A(i,i)*A(j_new,j_new));
             % neighbours belonging to previous pairs
             nbj=setdiff(find(A(j_new,:)),[j_new i]);                   
             nbj=nbj(find(isdetermined(nbj)>0));
             nbj=isdetermined(nbj);      
             % almost as good as the previous choice
             if rnb_new>=rnb*0.99
                if ~isneighbour
                   rnb=rnb_new;
                   j=j_new;
                   if ~isempty(nbj)
                      isneighbour=-1;
                   end
                else % the current choice already refers to a neighbour
                     % of at least one previous pair
                   if ~issameneighbour & ~isempty(nbj)
                      rnb=rnb_new;
                      j=j_new;
                      if ~isempty(intersect(nbj,nbi))
                         issameneighbour=-1;
                      end                      
                   end
                end
             end % if
          end % for
          % choose j as second member
          P{p}=[P{p} j]; 
          if nargin==2
             gplot(1,coord(j,:),'ob');
             gplot(A(P{p},P{p}),coord(P{p},:),'-b');
             axis([xmin xmax ymin ymax]);
             pause(.001);
          end % if
          % remove j from the list of neighbours
          nb(1)=nb(end);
          nb(end)=[];
          % remove j from the list of undetermined nodes
          posj=-isdetermined(j);
          ndetermined=ndetermined+1;
          if posj<=nnbp
             % move final neighbouring node to the front
             undetermined(posj)=undetermined(nnbp);
             isdetermined(undetermined(posj))=-posj;
             % move last node to position nnbp
             undetermined(nnbp)=undetermined(end);
             undetermined(end)=[];
             if nnbp<=length(undetermined)
                isdetermined(undetermined(nnbp))=-nnbp;
             end % i
             nnbp=nnbp-1;
          else
             undetermined(posj)=undetermined(end);
             undetermined(end)=[];
             if posj<=length(undetermined)
                isdetermined(undetermined(posj))=-posj;
             end % if
          end % if  
          isdetermined(j)=p;
          % set of undetermined neighbours of i
          nb=find(A(j,:));
          nb=nb(find(isdetermined(nb)<0));
          % shuffle neighbours of j to the end of list 'undetermined'
          for k=nb
              % swap entries
              posk=-isdetermined(k);
              if posk>nnbp
                 nnbp=nnbp+1;
                 undetermined(posk)=undetermined(nnbp);
                 undetermined(nnbp)=k;
                 isdetermined(undetermined(posk))=-posk;
                 isdetermined(k)=-nnbp;             
              end
          end % for k
      end % if-else
      % P{p}, ndetermined, pause
end % while
% ---------------   END first step: building pairs   ----------------------


% -----------------   second step: building groups   -----------------------
% reduce n to the number of pairs
n=p;
% set of pairs
G=cell(1,n); 
% how many pairs do exist
p=0;
ndeterminedp=0;
% list of undetermined nodes
undeterminedp=1:n;
% list of check marks. Let k=isdetermined(i). If k<0, then i has not been 
% determined yet. In this case it ist number |k| in the list of undetermined
% nodes. If k>0, then i is a member of group k
isdeterminedp=-undeterminedp;

% as long as pairs are available
while ndeterminedp<n
      p=p+1;
      % choose an undetermined pair i
      i=undeterminedp(1);
      % list might be inconsistent
      while isdeterminedp(i)>0
            undeterminedp(1)=undeterminedp(end);
            undeterminedp(end)=[];
            i=undeterminedp(1);
      end % while
      % choose the first subset
      G{p}=P{i};
      if nargin==2
         gplot(A(G{p},G{p}),coord(G{p},:),'og');
         axis([xmin xmax ymin ymax]);
         pause(.001);
      end
      % remove i from the list of undetermined pairs
      isdeterminedp(i)=p;
      ndeterminedp=ndeterminedp+1;
      undeterminedp(1)=undeterminedp(end);
      undeterminedp(end)=[];
      if ~isempty(undeterminedp)
         isdeterminedp(undeterminedp(1))=-1;
      end % if

      % set of undetermined neighbouring pairs of P{i}
      nb=[];
      for ii=P{i}
          nbii=find(A(ii,:));
          for jj=nbii
              pair=isdetermined(jj);
              if isdeterminedp(pair)<0
                 nb=union(nb,pair);
              end
          end % for
      end % for
      % i is a singleton
      if isempty(nb)
         % do nothing
      else
         % compute number of common edges between P{i} and neighbours
         mm=-1;
         for jj=1:length(nb)
             j_new=nb(jj);
             mm_new=nnz(A(P{j_new},P{i}));
             if mm_new>mm
                j=j_new;
                mm=mm_new;
             end
         end % for j
         % choose the second subset
         G{p}=[G{p} P{j}];

         if nargin==2
            gplot(A(G{p},G{p}),coord(G{p},:),'og');
            gplot(A(G{p},G{p}),coord(G{p},:),'-g');
            axis([xmin xmax ymin ymax]);
            pause(.001);
         end
         % remove j from the list of undetermined pairs
         pos=-isdeterminedp(j);
         isdeterminedp(j)=p;
         ndeterminedp=ndeterminedp+1;
         undeterminedp(pos)=undeterminedp(end);
         undeterminedp(end)=[];
         if pos<=length(undeterminedp)
            isdeterminedp(undeterminedp(pos))=-pos;
         end % if
         
      end % if-else 
end % while
% ---------------   END second step: building groups   ---------------------

% Third step: define prolongation operator
P=sparse(size(A,1),p);
for i=1:p
    % choose the first node in the list as coarse grid position
    C(i)=G{i}(1);
    % define prolongation
    P(G{i},i)=1;
end % for i
 