function [A,beta] = COLHOUSE(A,v,beta)
% [A,beta] = COLHOUSE(A,v,beta)
%
% A is multiplied by a Householder transformation from the right
% A := A * (I - (2/v'*v) * v*v') 
% Alternatives:  [A,beta]=COLHOUSE(A,v)        parameter 'beta' will be 
%                                              computed on the fly. 
%                 A      =COLHOUSE(A,v,beta)   parameter 'beta' will not be 
%                                              computed but prescribed.
% When calling  'ROWHOUSE/COLHOUSE/SYMHOUSE' multiple times it is quite useful
% to compute 'beta' only once by '[A,beta]=ROWHOUSE(A,v)', 
% '[A,beta]=COLHOUSE(A,v)' or '[A,beta]=SYMHOUSE(A,v)' and for any next call
% with the SAME 'v' only to call 'A=ROWHOUSE(A,v,beta), A=COLHOUSE(A,v,beta),
% A=SYMHOUSE(A,v,beta)' with fixed beta.
%
% flops for an  n x m matrix A:
% 4mn+2n-1 + 1 Div. (with calculation of beta), 
% 4mn               (without calculation of beta)

if (nargin==2)
   beta = -2/ (v'*v);
end % if
w = beta * (A * v);
A = A + w * v';
