function U=ichol(A)
% function U=ichol(A)
%
% produces for a symmetric M-matrix an incomplete modified Cholesky 
% decomposition, such that A=U'*U-E, where E is some nonnegative error matrix
% The factor U corresponds to the upper triangular part of the decomposition.
% The fill-in of U is the same as the fill-in of A. The diagonal of U is 
% modified such that the row sums of U'U and A coincide
modify=-1;

[n,n]=size(A);

U=triu(A);
for i=1:n-1
    U(i,i)=sqrt(U(i,i));
    U(i,i+1:n)=U(i,i)\U(i,i+1:n);

    % unmodified part of the ILU
    % for all rows where may exist fill-in 
    [I,J]=find(U(i,i+1:n));
    v=i+J;
    for j=v
	% for any column index which is already nonzero, update components
	[I,K]=find(U(j,j+1:n).*U(i,j+1:n));
	K=j+K;
        if (length(K)>0)
   	   U(j,K)=U(j,K)-U(i,j)'*U(i,K);
        end; % if
    end; % for

    % modify diagonal part
    if (modify~=0)
       for j=i+J
	   % for any column index which is zero, update diagonal entry
           [I,K]=find(([U(i+J(1):j-1,j)' U(j,j:n)]==0).*U(i,i+J(1):n));
	   K=[i+J(1)-1+K; j];
%           while (U(j,j)-U(i,j)'*sum(U(i,K))<=eps & length(K)>0)
%	         [y,l]=min(abs(U(i,K)));
%                 K(l)=[];
%           end;
           if (length(K)>0)
	      U(j,j)=U(j,j)-U(i,j)'*sum(U(i,K));
           end; % if
       end; % for
    else
       for j=i+J
	   U(j,j)=U(j,j)-U(i,j)'*U(i,j);
       end; % for
    end; % if
end; % for
U(n,n)=sqrt(U(n,n));
