function A=lap2d(n,fun)
% A=lap2d(n,fun)
%
% generate 2-dimensional Laplacian operator for the problem 
% -div(a(x,y)grad(y))=f in [0,1] and Dirichlet boundary cond. 
% use standard finite volume approach. 
%
% ---------------------------------------------------------
%
% Input
% -----
% fun    function a(x,y)
% n^2    size of the resulting system
% 
% ------------------------------------
%
% Output
% ------
% A      corresponding linear operator
h=1/(n+1);
omega=(1/2)*h:h:1;
a=feval(fun,omega,omega');
N=n*n;

% initialization, values will be overwritten, but the memory is allocated
A=sparse([],[],[],N,N,5*N);



i=1;
j=1;
jj=i;

% lower left corner
p=jj;
A(p,p)=A(p,p)+a(i,j);
jj=jj+n;

% left boundary
for j=2:n
    p=[jj-n jj];
    b=-a(i,j)/2;
    A(p,p)=A(p,p) + [a(i,j)    b;...
                        b   a(i,j)];
    jj=jj+n;		      
end % for
j=n+1;

% upper left corner
p=jj-n;
A(p,p)=A(p,p)+a(i,j);



for i=2:n
    j=1;
    jj=i;

    % lower boundary
    p=[jj-1 jj];
    b=-a(i,j)/2;
    A(p,p)=A(p,p) + [a(i,j)    b;...
                        b   a(i,j)];
    jj=jj+n;		      
		      
    % inner cells
    for j=2:n
        p=[jj-n-1 jj-n jj-1 jj];
	b=-a(i,j)/2;
	A(p,p)=A(p,p) + [a(i,j)    b      b     0;...
		            b   a(i,j)    0     b;...
			    b      0   a(i,j)   b;...
			    0      b      b   a(i,j)];
        jj=jj+n;		      
    end % for
    j=n+1;

    % upper boundary
    p=[jj-n-1 jj-n];
    b=-a(i,j)/2;
    A(p,p)=A(p,p) + [a(i,j)    b;...
                        b   a(i,j)];
end % for
i=n+1;



j=1;
jj=i;

% lower right corner
p=jj-1;
A(p,p)=A(p,p)+a(i,j);
jj=jj+n;

% right boundary
for j=2:n
    p=[jj-n-1 jj-1];
    b=-a(i,j)/2;
    A(p,p)=A(p,p) + [a(i,j)    b;...
                        b   a(i,j)];
    jj=jj+n;		      
end % for
j=n+1;

% upper right corner
p=jj-n-1;
A(p,p)=A(p,p)+a(i,j);
